package com.printer.sdk.android.update;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.printer.bluetooth.android.BluetoothPrinter;
import com.printer.bluetooth.android.PrinterType;
import com.printer.sdk.android.update.util.PrintUtils;

public class PrinterSDKDemo_Plus extends Activity implements View.OnClickListener{
	private final static String TAG = "PrinterSDKDemo_Plus";
	
	private Button openConnect;
	private Button printImage;
	private Button printCustomImage;
	
	private Button printText;
	private Button printTable;
	private Button printTitle;
	
	private Button printNote;
	private Button printBarCode;
	
	private RadioButton paperWidth_58;
	private RadioButton paperWidth_70;
	
	private RadioButton printer_type_remin;
	private RadioButton printer_type_zhenda;
	
	private TextView mTitle;
	private EditText mUserInputDeviceName;
	private EditText mBarCodeNum;
	
	// Name of the connected device
    private String mConnectedDeviceName = null;
	// Local Bluetooth adapter
    private BluetoothAdapter mBluetoothAdapter = null;
    public static BluetoothPrinter mPrinter;
    
    // Intent request codes
    private static final int REQUEST_CONNECT_DEVICE = 1;
    private static final int REQUEST_ENABLE_BT = 2;
    
    //menu context
    private static final int MENU_CONNECT = 0;
    private static final int MENU_DISCONNECT = 1;
    
    private Context mContext;
    private IntentFilter boundFilter = new IntentFilter(BluetoothDevice.ACTION_BOND_STATE_CHANGED);
    private BluetoothDevice currentDevice;
    
    private boolean isConnected;
    private boolean isConnecting;
    private boolean hasRegisteredBoundReceiver;
    
    //ͼ
    private List<String> list = new ArrayList<String>();
    private Spinner barCodeSpinner;
    private ArrayAdapter<String> adapter;
    
    //
    private List<String> encodingTypeList = new ArrayList<String>();
    private Spinner encodingTypeSpinner;
    private ArrayAdapter<String> encodingTypeAdapter;
    private String encodeType;
    
    private boolean is_58mm;
    private boolean is_remin;
    
    private Button xdjaBTN;
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_CUSTOM_TITLE);
		setContentView(R.layout.main);
		getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE, R.layout.custom_title);
		mContext = this;
		initView();
		// Get local Bluetooth adapter
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        // If the adapter is null, then Bluetooth is not supported
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, R.string.bt_not_enabled, Toast.LENGTH_LONG).show();
            finish();
        }
        registerReceiver(mStateReceiver, new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED));
	}
	
	@Override
	protected void onResume() {
		// TODO Auto-generated method stub
		super.onResume();
		if (!mBluetoothAdapter.isEnabled()) {
            Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
        }
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		try {
			mPrinter.closeConnection();
			isConnected = false;
			unregisterReceiver(mStateReceiver);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		android.os.Process.killProcess(android.os.Process.myPid());
	}
	
    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
    	menu.clear();
    	if(isConnecting){
    		return false;
    	}
    	if(isConnected){
    		menu.add(0, MENU_DISCONNECT, 0, R.string.disconnect).setIcon(android.R.drawable.ic_media_ff);
    	}else{
    		menu.add(0, MENU_CONNECT, 0, R.string.connect).setIcon(android.R.drawable.ic_media_play);
    	}
        return true;
    }

	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case MENU_CONNECT:
            // Launch the DeviceListActivity to see devices and do scan
            Intent serverIntent = new Intent(this, DeviceListActivity.class);
            startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
            return true;
        case MENU_DISCONNECT:
            // disconnect
        	mPrinter.closeConnection();
            return true;
        }
        return false;
    }
    
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		// TODO Auto-generated method stub
		switch (requestCode) {
        case REQUEST_CONNECT_DEVICE:
            // When DeviceListActivity returns with a device to connect
            if (resultCode == Activity.RESULT_OK) {
                // Get the device MAC address
                String address = data.getExtras().getString(DeviceListActivity.EXTRA_DEVICE_ADDRESS);
                // Get the BLuetoothDevice object
                BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
                mConnectedDeviceName = device.getName();
                Log.i(TAG, "connected device name is : " + mConnectedDeviceName + ", address is : " + address);
                Log.i(TAG, "device.getBondState() is : " + device.getBondState());
                
            	if (device.getBondState() == BluetoothDevice.BOND_NONE) { 
            		Log.i(TAG, "device.getBondState() is BluetoothDevice.BOND_NONE");
            		currentDevice = device;
            		registerReceiver(boundDeviceReceiver, boundFilter);
            		hasRegisteredBoundReceiver = true;
                    //use invoke method to call BluetoothDevice.createBond(BluetoothDevice remoteDevice); 
					try {
						//Method setPinMethod = BluetoothDevice.class.getMethod("setPin");
						//setPinMethod.invoke(device, 1234);
						Method createBondMethod = BluetoothDevice.class.getMethod("createBond");
						createBondMethod.invoke(device);
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
                     
                }else if(device.getBondState() == BluetoothDevice.BOND_BONDED){ 
                	Log.i(TAG, "device.getBondState() is BluetoothDevice.BOND_BONDED, open connect");
                    // Attempt to connect to the device
                	initPrinter(device);
                } 
            }
            break;
        case REQUEST_ENABLE_BT:
            // When the request to enable Bluetooth returns
            if (resultCode != Activity.RESULT_OK){
                // User did not enable Bluetooth or an error occured
                Toast.makeText(this, R.string.bt_not_enabled, Toast.LENGTH_SHORT).show();
                finish();
            }
        }
	}
	
	// The Handler that gets information back from the bluetooth printer.
    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
        	Log.i(TAG, "msg.what is : " + msg.what);
            switch (msg.what) {
            case BluetoothPrinter.Handler_Connect_Connecting:
            	isConnecting = true;
            	mTitle.setText(R.string.title_connecting);
            	Toast.makeText(getApplicationContext(), R.string.bt_connecting,Toast.LENGTH_SHORT).show();
            	break;
            case BluetoothPrinter.Handler_Connect_Success:
            	isConnected = true;
            	isConnecting = false;
            	mTitle.setText(getString(R.string.title_connected) + ": "+ mPrinter.getPrinterName());
            	Toast.makeText(getApplicationContext(), R.string.bt_connect_success,Toast.LENGTH_SHORT).show();
            	break;
            case BluetoothPrinter.Handler_Connect_Failed:
            	isConnected = false;
            	isConnecting = false;
            	mTitle.setText(R.string.title_not_connected);
            	Toast.makeText(getApplicationContext(), R.string.bt_connect_failed, Toast.LENGTH_SHORT).show();
            	break;
	        case BluetoothPrinter.Handler_Connect_Closed:
	        	isConnected = false;
	        	isConnecting = false;
	        	mTitle.setText(R.string.title_not_connected);
	        	Toast.makeText(getApplicationContext(), R.string.bt_connect_closed, Toast.LENGTH_SHORT).show();
	        	break;
	        }
            setButtonState();
        }
    };

	private void initView(){
		openConnect = (Button) this.findViewById(R.id.btnOpen);
		openConnect.setOnClickListener(this);
		
		printText = (Button) this.findViewById(R.id.btnPrintText);
		printText.setOnClickListener(this);
		
		printTable = (Button) this.findViewById(R.id.btnPrintTable);
		printTable.setOnClickListener(this);
		
		printImage = (Button) this.findViewById(R.id.btnPrintImage);
		printImage.setOnClickListener(this);
		
		printCustomImage = (Button) this.findViewById(R.id.btnPrintCustomImage);
		printCustomImage.setOnClickListener(this);
		
		printTitle = (Button) this.findViewById(R.id.btnPrintTitle);
		printTitle.setOnClickListener(this);
		
		printNote = (Button) this.findViewById(R.id.btnPrintNote);
		printNote.setOnClickListener(this);
		
		xdjaBTN = (Button)this.findViewById(R.id.btnZZXDJA);
		xdjaBTN.setOnClickListener(this);
		
		printBarCode = (Button) this.findViewById(R.id.btnPrintBarCode);
		printBarCode.setOnClickListener(this);
		
		paperWidth_58 = (RadioButton)findViewById(R.id.width_58mm);
		paperWidth_58.setOnClickListener(this);
		is_58mm = paperWidth_58.isChecked();
		
		paperWidth_70 = (RadioButton)findViewById(R.id.width_70mm);
		paperWidth_70.setOnClickListener(this);
		
		printer_type_remin = (RadioButton)findViewById(R.id.type_remin);
		printer_type_remin.setOnClickListener(this);
		is_remin = printer_type_remin.isChecked();
		
		printer_type_zhenda = (RadioButton)findViewById(R.id.type_zhenda);
		printer_type_zhenda.setOnClickListener(this);
		
		// Set up the custom title
        mTitle = (TextView) findViewById(R.id.title_right_text);
        mUserInputDeviceName = (EditText)findViewById(R.id.printerName);
        mBarCodeNum = (EditText)findViewById(R.id.bar_code_num);
        
        //뷽ʽ
        encodingTypeSpinner = (Spinner)findViewById(R.id.spinner_encoding_type);
        encodingTypeList.add("GBK");
        encodingTypeList.add("BIG5");
        encodingTypeList.add("UTF-8");
        encodingTypeList.add("GB2312");
        
      //ڶΪбһõǰ涨list      
        encodingTypeAdapter = new ArrayAdapter<String>(this,android.R.layout.simple_spinner_item, encodingTypeList);     
        //ΪбʱĲ˵ʽ      
        encodingTypeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);     
        //Ĳӵб      
        encodingTypeSpinner.setAdapter(encodingTypeAdapter);     
        //岽Ϊбø¼ӦӦ˵ѡ      
        encodingTypeSpinner.setOnItemSelectedListener(new Spinner.OnItemSelectedListener(){     
            @SuppressWarnings("unchecked")  
            public void onItemSelected(AdapterView arg0, View arg1, int arg2, long arg3) {     
                // TODO Auto-generated method stub      
                /* ѡmySpinner ֵmyTextView */    
                //myTextView.setText("ѡǣ"+ adapter.getItem(arg2));
            	encodeType = encodingTypeAdapter.getItem(arg2);
            	if(mPrinter != null)
            	{
            		mPrinter.setEncoding(encodeType);
            	}
                /* mySpinner ʾ*/    
                arg0.setVisibility(View.VISIBLE);     
            }     
            @SuppressWarnings("unchecked")  
            public void onNothingSelected(AdapterView arg0) {     
                // TODO Auto-generated method stub      
                //myTextView.setText("NONE");     
                arg0.setVisibility(View.VISIBLE);     
            }     
        });    
        
        
		//Ͳ-------start
        barCodeSpinner = (Spinner)findViewById(R.id.spinner_barcode_type);
        //һ: list
        list.add("UPC_A");     
        list.add("UPC_E");     
        list.add("JAN13(EAN13)");     
        list.add("JAN8(EAN8)");     
        list.add("CODE39"); 
        list.add("ITF"); 
        list.add("CODEBAR"); 
        list.add("CODE93"); 
        list.add("CODE128"); 
        list.add("PDF417"); 
        list.add("DATAMATRIX"); 
        list.add("QRCODE"); 
        
        //ڶΪбһõǰ涨list      
        adapter = new ArrayAdapter<String>(this,android.R.layout.simple_spinner_item, list);     
        //ΪбʱĲ˵ʽ      
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);     
        //Ĳӵб      
        barCodeSpinner.setAdapter(adapter);     
        //岽Ϊбø¼ӦӦ˵ѡ      
        barCodeSpinner.setOnItemSelectedListener(new Spinner.OnItemSelectedListener(){     
            @SuppressWarnings("unchecked")  
            public void onItemSelected(AdapterView arg0, View arg1, int arg2, long arg3) {     
                // TODO Auto-generated method stub      
                /* ѡmySpinner ֵmyTextView */    
                //myTextView.setText("ѡǣ"+ adapter.getItem(arg2));
            	String item = adapter.getItem(arg2);
            	if(item.equals("UPC_A") || item.equals("UPC_E") || item.equals("JAN13(EAN13)")){
            		mBarCodeNum.setText("123456789012");
            	}else if(item.equals("CODE128")){
            		mBarCodeNum.setText("No.123456");
            	}else if(item.equals("JAN8(EAN8)")){
            		mBarCodeNum.setText("1234567");
            	}else if(item.equals("PDF417") || item.equals("DATAMATRIX") || item.equals("QRCODE"))
            	{
            		mBarCodeNum.setText("123,45,67,89,01");
            	}else{
            		mBarCodeNum.setText("123456");
            	}
                /* mySpinner ʾ*/    
                arg0.setVisibility(View.VISIBLE);     
            }     
            @SuppressWarnings("unchecked")  
            public void onNothingSelected(AdapterView arg0) {     
                // TODO Auto-generated method stub      
                //myTextView.setText("NONE");     
                arg0.setVisibility(View.VISIBLE);     
            }     
        });     
        /*˵ѡ¼*/    
//        barCodeSpinner.setOnTouchListener(new Spinner.OnTouchListener(){     
//            public boolean onTouch(View v, MotionEvent event) {     
//                // TODO Auto-generated method stub      
//                /* mySpinner أҲԣԼ*/    
//                v.setVisibility(View.INVISIBLE);     
//                return false;     
//            }     
//        });     
//        /*˵ѡı¼*/    
//        barCodeSpinner.setOnFocusChangeListener(new Spinner.OnFocusChangeListener(){     
//        public void onFocusChange(View v, boolean hasFocus) {     
//        // TODO Auto-generated method stub      
//            v.setVisibility(View.VISIBLE);     
//        }     
//        });     
        //Ͳ-------end
        
        setButtonState();
		
	}
	
	private void setButtonState(){
		openConnect.setText(isConnected ? R.string.disconnect : R.string.connect);
		openConnect.setEnabled(!isConnecting);
		printTable.setEnabled(isConnected);
		
		printCustomImage.setEnabled(isConnected);
		printText.setEnabled(isConnected);
		printImage.setEnabled(isConnected);
		
		printNote.setEnabled(isConnected);
		printTitle.setEnabled(isConnected);
		mBarCodeNum.setEnabled(isConnected);
		
		printBarCode.setEnabled(isConnected);
		
		printer_type_remin.setEnabled(isConnected);
		printer_type_zhenda.setEnabled(isConnected);
		
		paperWidth_58.setEnabled(isConnected);
		paperWidth_70.setEnabled(isConnected);
		encodingTypeSpinner.setEnabled(isConnected);
		barCodeSpinner.setEnabled(isConnected);
		
		xdjaBTN.setEnabled(isConnected);
		xdjaBTN.setVisibility(View.GONE);
	}
	
	// receive the state change of the bluetooth.
	private BroadcastReceiver mStateReceiver = new BroadcastReceiver() { 
		public void onReceive(Context context, Intent intent) { 
			switch (mBluetoothAdapter.getState()) {
			case BluetoothAdapter.STATE_ON:
				Toast.makeText(mContext, R.string.bt_has_on, Toast.LENGTH_SHORT).show();
				break;
			case BluetoothAdapter.STATE_OFF:
				Toast.makeText(mContext, R.string.bt_has_off, Toast.LENGTH_SHORT).show();
				isConnected = false;
				break;
			default:
				break;
			}
		}
	};
	
	// receive bound broadcast to open connect.
	private BroadcastReceiver boundDeviceReceiver = new BroadcastReceiver() { 
        public void onReceive(Context context, Intent intent) { 
            String action = intent.getAction(); 
            BluetoothDevice device = null; 
            if(BluetoothDevice.ACTION_BOND_STATE_CHANGED.equals(action)){ 
                device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE); 
                if(!currentDevice.equals(device)){
                	return;
                }
                switch (device.getBondState()) { 
	                case BluetoothDevice.BOND_BONDING: 
	                    Log.i(TAG, "bounding......"); 
	                    break; 
	                case BluetoothDevice.BOND_BONDED: 
	                    Log.i(TAG, "bound success"); 
	                    // if bound success, auto init BluetoothPrinter. open connect.
	                    if(hasRegisteredBoundReceiver){
	                    	unregisterReceiver(boundDeviceReceiver);
	                    	hasRegisteredBoundReceiver = false;
	                    }
	                    initPrinter(device);
	                    break; 
	                case BluetoothDevice.BOND_NONE: 
	                	if(hasRegisteredBoundReceiver){
	                    	unregisterReceiver(boundDeviceReceiver);
	                    	hasRegisteredBoundReceiver = false;
	                    }
	                    Log.i(TAG, "bound cancel"); 
                default: 
                    break; 
                } 
            } 
        } 
    }; 
    
    // use device to init printer.
    private void initPrinter(BluetoothDevice device){
    	mPrinter = new BluetoothPrinter(device);
    	
    	if(is_remin)
    	{
    		mPrinter.setCurrentPrintType(is_58mm ? PrinterType.TIII : PrinterType.T9);
    	}else{
    		mPrinter.setCurrentPrintType(PrinterType.T5);
    	}
    	//set handler for receive message of connect state from sdk.
        mPrinter.setHandler(mHandler); 
        mPrinter.openConnection();
        
        mPrinter.setEncoding(encodeType);
    }
    
    // use device to init printer.
    @SuppressWarnings("deprecation")
	private void initPrinter(String deviceName){
    	mPrinter = new BluetoothPrinter(deviceName);
    	if(mPrinter.isPrinterNull()){
    		Toast.makeText(mContext, getString(R.string.no_bounded_device, deviceName), Toast.LENGTH_LONG).show();
    		return;
    	}
    	if(is_remin)
    	{
    		mPrinter.setCurrentPrintType(is_58mm ? PrinterType.TIII : PrinterType.T9);
    	}else{
    		mPrinter.setCurrentPrintType(PrinterType.T5);
    	}
    	//set handler for receive message of connect state from sdk.
    	mPrinter.setHandler(mHandler); 
    	mPrinter.openConnection();
    	mPrinter.setEncoding(encodeType);
//    	String country = Locale.getDefault().getCountry();
//    	Log.i(TAG, "local country is: " + country);
//    	if ("TW".equalsIgnoreCase(country)) {
//    		mPrinter.setEncoding("BIG5");
//		}
    }
    
	@Override
	public void onClick(View view) {
		// TODO Auto-generated method stub
		if (view == openConnect) {
			if(isConnected){
				mPrinter.closeConnection();
			}else{
				initPrinter(mUserInputDeviceName.getText().toString());
			}
		} else if (view == printText) {
			Intent edit_intent = new Intent(mContext, PrinterTextEdit.class);
			startActivity(edit_intent);
		} else if (view == printTable) {
			PrintUtils.printTable(mContext.getResources(), mPrinter);
		} else if (view == printImage) {
			if (mPrinter.getCurrentPrintType() == PrinterType.T5){
				PrintUtils.printDrawToPOS76(mContext.getResources(), mPrinter);
			}else{
				PrintUtils.printImage(mContext.getResources(), mPrinter);
			}
		} else if (view == printCustomImage) {
			PrintUtils.printCustomImage(mContext.getResources(), mPrinter);
		} else if (view == printTitle) {
			PrintUtils.printTitle(mContext.getResources(), mPrinter);
		} else if (view == printNote){
			PrintUtils.printNote(mContext.getResources(), mPrinter);
		} else if (view == printBarCode){
			PrintUtils.printBarCode(mPrinter, mBarCodeNum.getText().toString(), barCodeSpinner.getSelectedItem().toString());
		} else if(view == xdjaBTN){
			PrintUtils.printXDJANote(mContext.getResources(),mPrinter);
		} else if (view == paperWidth_58){
			is_58mm = true;
			if(is_remin)
	    	{
	    		mPrinter.setCurrentPrintType(PrinterType.TIII);
	    	}else{
	    		mPrinter.setCurrentPrintType(PrinterType.T5);
	    	}
		} else if (view == paperWidth_70){
			is_58mm = false;
			if(is_remin)
	    	{
	    		mPrinter.setCurrentPrintType(PrinterType.T9);
	    	}else{
	    		mPrinter.setCurrentPrintType(PrinterType.T5);
	    	}
		} else if (view == printer_type_remin){
			is_remin = true;
			mPrinter.setCurrentPrintType(PrinterType.TIII);
			if(is_58mm)
	    	{
	    		mPrinter.setCurrentPrintType(PrinterType.TIII);
	    	}else{
	    		mPrinter.setCurrentPrintType(PrinterType.T9);
	    	}
		} else if (view == printer_type_zhenda){
			is_remin = false;
			if(is_58mm)
	    	{
	    		mPrinter.setCurrentPrintType(PrinterType.T5);
	    	}else{
	    		mPrinter.setCurrentPrintType(PrinterType.T5);
	    	}
		}

	}

}